package ctbrec.sites.flirt4free;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ctbrec.Config;
import ctbrec.Model;
import ctbrec.io.HtmlParser;
import ctbrec.io.HttpClient;
import ctbrec.io.HttpException;
import ctbrec.sites.AbstractSite;
import okhttp3.Request;
import okhttp3.Response;

public class Flirt4Free extends AbstractSite {

    private static final transient Logger LOG = LoggerFactory.getLogger(Flirt4Free.class);
    public static final String BASE_URI = "https://www.flirt4free.com";
    private HttpClient httpClient;

    @Override
    public String getName() {
        return "Flirt4Free";
    }

    @Override
    public String getBaseUrl() {
        return BASE_URI;
    }

    @Override
    public String getAffiliateLink() {
        return BASE_URI;
    }

    @Override
    public String getBuyTokensLink() {
        return BASE_URI;
    }

    @Override
    public Model createModel(String name) {
        Flirt4FreeModel model = new Flirt4FreeModel();
        model.setName(name);
        model.setUrl(getBaseUrl() + "/rooms/" + name + '/');
        model.setSite(this);
        return model;
    }

    @Override
    public Double getTokenBalance() throws IOException {
        return 0d;
        //        if (!credentialsAvailable()) {
        //            throw new IOException("Account settings not available");
        //        }
        //
        //        String username = Config.getInstance().getSettings().camsodaUsername;
        //        String url = BASE_URI + "/api/v1/user/" + username;
        //        Request request = new Request.Builder().url(url).build();
        //        try(Response response = getHttpClient().execute(request)) {
        //            if(response.isSuccessful()) {
        //                JSONObject json = new JSONObject(response.body().string());
        //                if(json.has("user")) {
        //                    JSONObject user = json.getJSONObject("user");
        //                    if(user.has("tokens")) {
        //                        return (double) user.getInt("tokens");
        //                    }
        //                }
        //            } else {
        //                throw new HttpException(response.code(), response.message());
        //            }
        //        }
        //        throw new RuntimeException("Tokens not found in response");
    }

    @Override
    public synchronized boolean login() throws IOException {
        return credentialsAvailable() && getHttpClient().login();
    }

    @Override
    public HttpClient getHttpClient() {
        if(httpClient == null) {
            httpClient = new Flirt4FreeHttpClient();
        }
        return httpClient;
    }

    @Override
    public void init() throws IOException {
    }

    @Override
    public void shutdown() {
        if(httpClient != null) {
            httpClient.shutdown();
        }
    }

    @Override
    public boolean supportsTips() {
        return false;
    }

    @Override
    public boolean supportsFollow() {
        return false;
    }

    @Override
    public boolean supportsSearch() {
        return true;
    }

    @Override
    public List<Model> search(String q) throws IOException, InterruptedException {
        String url = BASE_URI + "/search/?query=" + URLEncoder.encode(q, "utf-8");
        Request req = new Request.Builder()
                .url(url)
                .addHeader("User-Agent", Config.getInstance().getSettings().httpUserAgent)
                .build();
        try(Response response = getHttpClient().execute(req)) {
            if(response.isSuccessful()) {
                List<Model> result = new ArrayList<>();
                String body = response.body().string();
                Elements modelLinks = HtmlParser.getTags(body, "form#advanced_search h4 a.common-link");
                for (Element link : modelLinks) {
                    String cellHtml = link.parent().parent().parent().html();
                    link.setBaseUri(BASE_URI);
                    String bioPage = link.absUrl("href");
                    Model model = createModelFromUrl(bioPage);
                    Element img = HtmlParser.getTag(cellHtml, "img[alt]");
                    model.setPreview(img.attr("src"));
                    model.setDisplayName(img.attr("alt"));
                    result.add(model);
                }
                return result;
            } else {
                throw new HttpException(response.code(), response.message());
            }
        }
    }

    @Override
    public boolean isSiteForModel(Model m) {
        return m instanceof Flirt4FreeModel;
    }

    @Override
    public boolean credentialsAvailable() {
        String username = Config.getInstance().getSettings().camsodaUsername;
        return username != null && !username.trim().isEmpty();
    }

    @Override
    public Model createModelFromUrl(String url) {
        String[] patterns = {
                "https?://(?:www\\.)?flirt4free.com/\\?model=(.*?)",
                "https?://(?:www\\.)?flirt4free.com/rooms/(.*?)/?",
                "https?://(?:www\\.)?flirt4free.com/models/bios/(.*?)/about.php"
        };
        for (String p : patterns) {
            Matcher m = Pattern.compile(p).matcher(url);
            if(m.matches()) {
                String modelName = m.group(1);
                return createModel(modelName);
            }
        }
        return super.createModelFromUrl(url);
    }
}
